#include<stdio.h>
#include<stdlib.h>
#include<string.h>
#include<mpi.h>
#include<hdf5.h>
#include "header.h"

#define HEADERSETNAME "HeaderOfStructure"
#define DATASETNAME "DataOfStructure"
typedef struct _header {
	int ng,ngz,nspace;
	float size,omep,omeplam,amax;
	float a,redshift;
	float epsilon;
	double pmass,r1kfact,r2kfact;
	double com2real,real2com,potentfact;
	int num_halo;
} header;
typedef struct _Menu_item{
	char *title;
	int offset;
	char *type;
} Menu_item;

Menu_item sim_data[] = {
    {"x(in mesh scale)",HOFFSET(basicparticletype,x),"H5T_NATIVE_FLOAT"},
    {"y(in mesh scale)",HOFFSET(basicparticletype,y),"H5T_NATIVE_FLOAT"},
    {"z(in mesh scale)",HOFFSET(basicparticletype,z),"H5T_NATIVE_FLOAT"},
    {"vx(in mesh scale)",HOFFSET(basicparticletype,vx),"H5T_NATIVE_FLOAT"},
    {"vy(in mesh scale)",HOFFSET(basicparticletype,vy),"H5T_NATIVE_FLOAT"},
    {"vz(in mesh scale)",HOFFSET(basicparticletype,vz),"H5T_NATIVE_FLOAT"},
    {"index",HOFFSET(basicparticletype,indx),"H5T_NATIVE_INT"},
	NULL
};
Menu_item inform[]={
	{"ng",HOFFSET(header,ng),"H5T_NATIVE_INT"},
	{"ngz",HOFFSET(header,ngz),"H5T_NATIVE_INT"},
	{"nspace",HOFFSET(header,nspace),"H5T_NATIVE_INT"},
	{"size",HOFFSET(header,size),"H5T_NATIVE_FLOAT"},
	{"omep",HOFFSET(header,omep),"H5T_NATIVE_FLOAT"},
	{"omeplam",HOFFSET(header,omeplam),"H5T_NATIVE_FLOAT"},
	{"amax",HOFFSET(header,amax),"H5T_NATIVE_FLOAT"},
	{"a",HOFFSET(header,a),"H5T_NATIVE_FLOAT"},
	{"redshift",HOFFSET(header,redshift),"H5T_NATIVE_FLOAT"},
	{"mass per particle",HOFFSET(header,pmass),"H5T_NATIVE_DOUBLE"},
	{"1'st kinetic energy factor",HOFFSET(header,r1kfact),"H5T_NATIVE_DOUBLE"},
	{"2nd kinetic energy factor",HOFFSET(header,r2kfact),"H5T_NATIVE_DOUBLE"},
	{"comoving to real space",HOFFSET(header,com2real),"H5T_NATIVE_DOUBLE"},
	{"real to comoving space",HOFFSET(header,real2com),"H5T_NATIVE_DOUBLE"},
	{"potential energy factor",HOFFSET(header,potentfact),"H5T_NATIVE_DOUBLE"},
	{"total number of halos",HOFFSET(header,num_halo),"H5T_NATIVE_INT"},
	NULL
};
void hdf5_insert(hid_t id,Menu_item *);
extern float amax,a,rng,size,hubble;
extern int ng,nspace;
extern float omep,omeplam,epsilon;
extern int nx,ny,nz;
extern double onesolarmass,pntmass,r1kineticfact,r2kineticfact;
extern double com2real,real2com,potentfact;
static hid_t file,group_id;
/* writing data */
extern lint mpeak;
extern lint *ptl2halonum;
/* variables for HDF file */
header sim_inform;
hid_t header_set,header_space;
hid_t header_id,data_id;
hid_t group_id_info,group_id_data;
herr_t status;
hsize_t dim;

hid_t particle_dataspace;
hid_t particle_dataset;
hid_t cparms;
hid_t particle_datatype;
hsize_t chunk_dims[] = {2};
hsize_t particle_size[] = {0};
hsize_t maxdims[] = {H5S_UNLIMITED};
/* total number of halos recorded */
static num_halos=0;
static total_num_halos=0,hstart=0,hfinal=0;
int INIT=0;
typedef struct _halostruct {
	int index;
	int start,final;
} halostruct;
halostruct *halo;
Menu_item inform_halo[] = {
	{"index",HOFFSET(halostruct,index),"H5T_NATIVE_INT"},
	{"start",HOFFSET(halostruct,start),"H5T_NATIVE_INT"},
	{"final",HOFFSET(halostruct,final),"H5T_NATIVE_INT"},
	NULL
};
#define NUM_HALO 1000000

void bin2ascii(struct PtlPos *r,struct PtlVel *vr,lint np);
void hdf_write_data(struct PtlPos *rx,struct PtlVel *rvx,lint *rindx,int rnp){
	basicparticletype *particle;
	lint wnp;
	int i,j,k;
	char dir1[7],dir2[6];
	hsize_t width[] = {0};
	hssize_t offset[] = {0};
	hid_t type,filespace;
	if(mpeak >0){
		num_halos++;
		particle = (basicparticletype *) malloc
			(sizeof(basicparticletype)*rnp);
		for(i=0;i<mpeak;i++){
			wnp = 0;
			for(j=0;j<rnp;j++)
				if(ptl2halonum[j] == i){
					particle[wnp].x = rx[j].x;
					particle[wnp].y = rx[j].y;
					particle[wnp].z = rx[j].z;
					particle[wnp].vx = rvx[j].vx;
					particle[wnp].vy = rvx[j].vy;
					particle[wnp].vz = rvx[j].vz;
					particle[wnp].indx = rindx[j];
					wnp++;
				}
			if(wnp != 0){
				(halo+total_num_halos)->index = num_halos;
				(halo+total_num_halos)->start = hfinal+1;
				(halo+total_num_halos)->final = hfinal+wnp;
				total_num_halos++;
				particle_size[0] = hfinal+wnp;
				status = H5Dextend(particle_dataset,particle_size);
				filespace = H5Dget_space(particle_dataset);
				offset[0] = hfinal;
				width[0] = wnp;
				status = H5Sselect_hyperslab(filespace,H5S_SELECT_SET,
						offset,NULL,width,NULL);
	
				particle_dataspace = H5Screate_simple(1,width,NULL);
	
				status = H5Dwrite(particle_dataset,particle_datatype,
						particle_dataspace,filespace,H5P_DEFAULT,particle);
	
				hfinal += wnp;
			}
		}
		free(particle);
	}
#ifdef CHECK_HALO
    bin2ascii(rx,rvx,rnp);
#endif
}
/* closing the output file */
void final_out_file(){
	/* setting the information of the simulation */
	sim_inform.ng = nx;
	sim_inform.ngz = nz;
	sim_inform.nspace = nspace;
	sim_inform.size = size;
	sim_inform.omep = omep;
	sim_inform.omeplam = omeplam;
	sim_inform.amax = amax;
	sim_inform.a = a;
	sim_inform.redshift = amax/a - 1.;
	sim_inform.epsilon = epsilon;
	sim_inform.pmass = pntmass * onesolarmass;
	sim_inform.r1kfact = r1kineticfact;
	sim_inform.r2kfact = r2kineticfact;
	sim_inform.com2real = com2real;
	sim_inform.real2com = real2com;
	sim_inform.potentfact = potentfact;
	sim_inform.num_halo = total_num_halos;
	/*   ùķ̼ ֿ Ķ͵ Ѵ.  */
	dim = 1;
	header_space = H5Screate_simple(1,&dim,NULL);
	group_id_info = H5Gcreate(file,"/SIMULATION RESULTS/INFORMATION",0);
	header_id = H5Tcreate (H5T_COMPOUND,sizeof(header));
	hdf5_insert(header_id,inform);
	header_set = H5Dcreate(group_id_info,"SIM_PARAMETERS",header_id,
			header_space,H5P_DEFAULT);
	status = H5Dwrite(header_set,header_id,H5S_ALL, H5S_ALL,
			H5P_DEFAULT,&sim_inform);
	H5Dclose(header_set);H5Tclose(header_id); H5Sclose(header_space);
	H5Gclose(group_id_info);
	/*   Ÿ鿡 ش Ϸ ȣ Ѵ.   */
	dim = total_num_halos;
	header_space = H5Screate_simple(1,&dim,NULL);
	group_id_info = H5Gcreate(file,"/SIMULATION RESULTS/INFORMATION/P_HEAD",0);
	header_id = H5Tcreate (H5T_COMPOUND,sizeof(halostruct));
	hdf5_insert(header_id,inform_halo);
	header_set = H5Dcreate(group_id_info,"HALO_DATA",header_id,
			header_space,H5P_DEFAULT);
	status = H5Dwrite(header_set,header_id,H5S_ALL, H5S_ALL,
			H5P_DEFAULT,halo);
	H5Dclose(header_set);H5Tclose(header_id); H5Sclose(header_space);
	H5Gclose(group_id_info);
	H5Gclose(group_id);
	/*   */
	H5Fclose(file);
	free(halo);
}
/* initializing and inserting the header information to the output file */
void initial_out_file(char *filename){
	INIT = 0;
	num_halos = 0;
	total_num_halos =hstart=hfinal= 0;
	fprintf(stdout,"initializing the header of output data file\n");
	file = H5Fcreate(filename,H5F_ACC_TRUNC, H5P_DEFAULT, H5P_DEFAULT);
	group_id = H5Gcreate(file,"/SIMULATION RESULTS",0);
	/*  */
	halo = (halostruct *)malloc(sizeof(halostruct)*NUM_HALO);
	/* setting for extendible data space for particle data */
	dim = 1;
	particle_dataspace = H5Screate_simple(1,&dim,maxdims);
	particle_datatype = H5Tcreate(H5T_COMPOUND,sizeof(basicparticletype));
	hdf5_insert(particle_datatype,sim_data);
	cparms = H5Pcreate(H5P_DATASET_CREATE);
	status = H5Pset_chunk (cparms,1,chunk_dims);
	particle_dataset = H5Dcreate(file,"/PARTICLE_DATA",particle_datatype,
			particle_dataspace,cparms);
}
void hdf5_insert(hid_t id,Menu_item *menu){
	int i=0;
	hid_t type;
	while(menu[i].title != NULL){
		if(strcmp(menu[i].type,"H5T_NATIVE_INT")==0)
			type = H5T_NATIVE_INT;
        else if(strcmp(menu[i].type,"H5T_NATIVE_FLOAT")==0)
            type = H5T_NATIVE_FLOAT;
        else if(strcmp(menu[i].type,"H5T_NATIVE_CHAR")==0)
            type = H5T_NATIVE_CHAR;
        else if(strcmp(menu[i].type,"H5T_NATIVE_LONG")==0)
            type = H5T_NATIVE_LONG;
        else if(strcmp(menu[i].type,"H5T_NATIVE_UINT")==0)
            type = H5T_NATIVE_UINT;
        else if(strcmp(menu[i].type,"H5T_NATIVE_DOUBLE")==0)
            type = H5T_NATIVE_DOUBLE;
        else {
            fprintf(stderr,"%s : ",menu[i].type);
            fprintf(stderr,"Not supported data type in HDF format\n");
            fprintf(stderr,"Exiting \n");
			MPI_Barrier(MPI_COMM_WORLD);
			MPI_Finalize();
            exit(0);
        }
        H5Tinsert(id,menu[i].title,menu[i].offset,type);
        i = i + 1;
    }
}


