#include<stdio.h>
#include<stdlib.h>
#include<stddef.h>
#include<string.h>
#include<math.h>
#include<mpi.h>
#include"Memory.h"
#include "tree.h"
#include "defs.h"
#include "header.h"
static int tidal_start=1;
float m_tidal[NUM_MASS],r_tidal[NUM_MASS];
typedef struct Halo{
	size_t np;
	float x,y,z,vx,vy,vz;
} Halo;
Halo halo;
basicparticletype *bp,*sbp,*rbp;

int myid,nid;
int shalonum,rhalonum;
lint *ptl2halonum;
int mpeak;
int ready=READY,writing=WRITING;
int snp,rnp;
FILE *wrp,*wbp;
/*                              */
float amax,a,rng,size,hubble;
int ng,nspace;
float omep,omepb,omeplam,epsilon;
int nx,ny,nz;
/*                              */
double onesolarmass=1.989E33L;
double com2real,real2com,potentfact;
double pntmass,r1kineticfact,r2kineticfact;
float acoeff[8];

void write_data(basicparticletype *,lint*,int);

int physical_parameters(void);
int subhalo_den(basicparticletype *, lint,lint *);
void pspline_(void);
void i_potent_spline(void);

int main(int argc, char *argv[]) {
	MPI_Status mstatus,cstatus;
	MPI_Request request;
	char filetag[80],filename[80];
	int i,j,snp,rnp,flag;
	int finish=-99999;
	int snend,rnend;
	int nstep;
	int src,dest;
	/*          */
	/*
	struct HaloPos *csx,*crx;
	struct HaloVel *csvx,*crvx;
	float *floatsx, *floatrx;
	int *swap;
	*/
	float tmp_tidal,tmp;
	/*          */
	FILE *fp,*rhfp,*rtidalf;
	char rheader[80],rfilename[80],wfilename[80];



	(void )MPI_Init(&argc,&argv);
	(void )MPI_Comm_size(MPI_COMM_WORLD,&nid);
	(void )MPI_Comm_rank(MPI_COMM_WORLD,&myid);

	if(argc != 2){
		if(myid == motherrank)
		fprintf(stderr,".... hfind.exe [nstep]\n");
		exit(1);
	}
	nstep = atoi(argv[1]);

	if(Make_Total_Memory()==0){
		fprintf(stderr,"P%d Error initializing mem %ldMB\n",myid,NMEG);
	}

	i_potent_spline();
	if(myid == motherrank && tidal_start == 1){
		fprintf(stderr,"reading tidal.dat");
		if((rtidalf = fopen("tidal.dat","r"))==NULL){
			fprintf(stderr,"Error opening tidal.dat\n");
			exit(9);
		}
			
		for(i=0;i<NUM_MASS;i++){
			fscanf(rtidalf,"%g %g %g\n",&m_tidal[i],&r_tidal[i],&tmp);
		}
		fclose(rtidalf);
		tidal_start = 0;
		fprintf(stderr,"read tidal.dat");
	}
	MPI_Bcast(m_tidal,NUM_MASS,MPI_FLOAT,motherrank,MPI_COMM_WORLD);
	MPI_Bcast(r_tidal,NUM_MASS,MPI_FLOAT,motherrank,MPI_COMM_WORLD);
	sprintf(rheader,"FoF_halo_cat.%.5d",nstep);
	if(myid ==0) {
		fprintf(stderr,"opening header file");
		rhfp = fopen(rheader,"r");
		{
			float bias;
			float astep,anow;
			int npower;
         	fread(&size,sizeof(float),1,rhfp);
		 	fread(&hubble,sizeof(float),1,rhfp);
			fread(&npower,sizeof(float),1,rhfp);
			fread(&omep,sizeof(float),1,rhfp);
			fread(&omepb,sizeof(float),1,rhfp);
			fread(&omeplam,sizeof(float),1,rhfp);
			fread(&bias,sizeof(float),1,rhfp);
			fread(&nx,sizeof(int),1,rhfp);
			fread(&nspace,sizeof(int),1,rhfp);
			fread(&amax,sizeof(float),1,rhfp);
			fread(&astep,sizeof(float),1,rhfp);
			fread(&anow,sizeof(float),1,rhfp);
			ng = nx;
			a = anow;
		}
		fprintf(stderr," well open header file\n");
	}
	MPI_Bcast(&ng,1,MPI_INT,motherrank,MPI_COMM_WORLD);
	MPI_Bcast(&nspace,1,MPI_INT,motherrank,MPI_COMM_WORLD);
	MPI_Bcast(&omep,1,MPI_FLOAT,motherrank,MPI_COMM_WORLD);
	MPI_Bcast(&omeplam,1,MPI_FLOAT,motherrank,MPI_COMM_WORLD);
	MPI_Bcast(&hubble,1,MPI_FLOAT,motherrank,MPI_COMM_WORLD);
	MPI_Bcast(&size,1,MPI_FLOAT,motherrank,MPI_COMM_WORLD);
	MPI_Bcast(&amax,1,MPI_FLOAT,motherrank,MPI_COMM_WORLD);
	MPI_Bcast(&a,1,MPI_FLOAT,motherrank,MPI_COMM_WORLD);
	if(myid==motherrank){
		char backfile[190];
		sprintf(backfile,"background_ptl%.5d.dat",nstep);
		wbp = fopen(backfile,"w");
	}
	rng = ng;
	nx = ny = nz = ng;
	epsilon = (nspace*0.1)*(nspace*0.1);

	acoeff[0] = 1.;
	(void) physical_parameters();
#ifdef DEBUG
	if(myid==0) fprintf(stderr,"Well calculating physical parameters size=%g omepb=%g potentfact=%g\n",
			size,omepb,potentfact);
#endif
#ifdef PMFORCE
    pspline_();
#endif
	sprintf(rfilename,"FoF_member_particle.%.5d",nstep);
	sprintf(wfilename,"HFIND.DATA.%.5d",nstep);
	if(myid==motherrank) {
		fprintf(stderr,"Opening %s file for output\n",wfilename);
		wrp = fopen(wfilename,"w");
	}
	snend = rnend = INIT_NP;
	shalonum = rhalonum = 0;
	sbp = (basicparticletype*)Malloc(sizeof(basicparticletype)*snend,PPTR(sbp));
	ptl2halonum = (lint *)Malloc(sizeof(lint)*INIT_NP,PPTR(ptl2halonum));
	if(myid == motherrank) {
		rbp = (basicparticletype*)Malloc(sizeof(basicparticletype)*INIT_NP,PPTR(rbp));
		fp = fopen(rfilename,"r");
		while(fread(&halo,sizeof(Halo),1,rhfp) == 1){
			snp = halo.np; snend = MAX(snend,snp);
			sbp = Realloc(sbp,sizeof(basicparticletype)*snend);
			fread(sbp,sizeof(basicparticletype),snp,fp);
			if(snp > 20) {
			/*
			if(snp == 8736) {
			if(snp == 14514) {
			if(snp == 393816) {
			if(snp == 31568) {
			if(snp == 14334) {
			if(snp == 4284) {
			if(snp == 31568) {
			if(snp > 50000) {
			if(snp == 18061) {
			if(snp == 262293) {
			*/
				fprintf(stdout,"P%d read %d particles\n",myid,snp);
				do {
					MPI_Probe(MPI_ANY_SOURCE,READY,MPI_COMM_WORLD,&mstatus);
					src = mstatus.MPI_SOURCE;
					dest = mstatus.MPI_SOURCE;
					MPI_Recv(&ready,1,MPI_INT,src,READY,MPI_COMM_WORLD,&cstatus);
					if(ready == READY){
						shalonum++;
						MPI_Send(&snp,1,MPI_INT,dest,NP_TAG, MPI_COMM_WORLD);
						MPI_Send(sbp,snp*sizeof(basicparticletype),MPI_BYTE,dest,R_TAG,MPI_COMM_WORLD);
						fprintf(stderr,"P0 sending to %d with %d particles\n",mstatus.MPI_SOURCE,snp); 
						fflush(stderr);
						/*
						MPI_Finalize();exit(99);
						*/
					}
					else {
						rhalonum++;
						MPI_Probe(src,NP_TAG,MPI_COMM_WORLD,&cstatus);
						MPI_Recv(&rnp,1,MPI_INT,src,NP_TAG,MPI_COMM_WORLD,&cstatus);
						fprintf(stderr,"receiving from %d with %d particles\n",
							src,rnp); fflush(stderr);
						if(rnp > rnend) {
							rnend = MAX(rnend,rnp);
							rbp = Realloc(rbp,sizeof(basicparticletype)*rnend);
							ptl2halonum = Realloc(ptl2halonum,sizeof(lint)*rnend);
						}
						MPI_Probe(src,R_TAG,MPI_COMM_WORLD,&cstatus);
						MPI_Recv(rbp,sizeof(basicparticletype)*rnp,MPI_BYTE,src,R_TAG,MPI_COMM_WORLD,&cstatus);
						MPI_Probe(src,PT2H_TAG,MPI_COMM_WORLD,&cstatus);
						MPI_Recv(ptl2halonum,rnp,MPI_INT,src, PT2H_TAG, MPI_COMM_WORLD,&cstatus);
						MPI_Probe(src,MPEAK_TAG,MPI_COMM_WORLD,&cstatus);
						MPI_Recv(&mpeak,1,MPI_INT,src,MPEAK_TAG, MPI_COMM_WORLD,&cstatus);
						write_data(rbp,ptl2halonum,rnp);
						printf("P0 %d received with mpeak=%d\n",rnp,mpeak);
					}
				} while(ready != READY); 
				if(rnend > 2*INIT_NP) rnend = 2*INIT_NP;
				if(snend > 2*INIT_NP) snend = 2*INIT_NP;
			}
		}
		fclose(fp);
exithere :
		printf("complete................... \n");
		fflush(stdout);
		/*    α׷ ģ. ⿡  Ϸ
		 *  ޾Ƽ  Ϸ  ƾ߸  ־ Ѵ. */
		j = 0;
		for(i=rhalonum+1;i<=shalonum;){
			MPI_Probe(MPI_ANY_SOURCE,READY,MPI_COMM_WORLD,&mstatus);
			MPI_Recv(&ready,1,MPI_INT,mstatus.MPI_SOURCE,READY,
					MPI_COMM_WORLD,&cstatus);
			if(ready == WRITING){
				i++;
				MPI_Probe(mstatus.MPI_SOURCE,NP_TAG,MPI_COMM_WORLD,&cstatus);
				MPI_Recv(&rnp,1,MPI_INT,mstatus.MPI_SOURCE,NP_TAG,MPI_COMM_WORLD, &cstatus);
				rnend = MAX(rnend,rnp);
				/*
				rx = Realloc(rx,sizeof(struct PtlPos)*rnend);
				rvx = Realloc(rvx,sizeof(struct PtlVel)*rnend);
				rindx = Realloc(rindx,sizeof(indxtype)*rnend);
				*/
				rbp = Realloc(rbp,sizeof(basicparticletype)*rnend);
				ptl2halonum = Realloc(ptl2halonum,sizeof(lint)*rnend);
				MPI_Probe(mstatus.MPI_SOURCE,R_TAG,MPI_COMM_WORLD, &cstatus);
				MPI_Recv(rbp,sizeof(basicparticletype)*rnp,MPI_BYTE,mstatus.MPI_SOURCE,R_TAG,MPI_COMM_WORLD,&cstatus);
				MPI_Probe(mstatus.MPI_SOURCE,PT2H_TAG,MPI_COMM_WORLD, &cstatus);
				MPI_Recv(ptl2halonum,rnp,MPI_INT,mstatus.MPI_SOURCE,PT2H_TAG, MPI_COMM_WORLD,&cstatus);
				MPI_Probe(mstatus.MPI_SOURCE,MPEAK_TAG,MPI_COMM_WORLD, &cstatus);
				MPI_Recv(&mpeak,1,MPI_INT,mstatus.MPI_SOURCE,MPEAK_TAG, MPI_COMM_WORLD,&cstatus);
				fprintf(stderr,"receiving from %d with %d particles\n",
							mstatus.MPI_SOURCE,rnp); fflush(stderr);
				write_data(rbp,ptl2halonum,rnp);
			}
			else {
				j++;
				MPI_Send(&finish,1,MPI_INT,mstatus.MPI_SOURCE, NP_TAG,MPI_COMM_WORLD);
			}
		}
		for(i=1;i<nid-j;i++){
			MPI_Recv(&ready,1,MPI_INT,MPI_ANY_SOURCE,READY, MPI_COMM_WORLD,&mstatus);
			MPI_Send(&finish,1,MPI_INT,mstatus.MPI_SOURCE, NP_TAG,MPI_COMM_WORLD);
		}
		fclose(wrp);
		fclose(wbp);
	}
	/* ٸ μ ⼭  Ѵ  */
	else {
		ready = READY;
		MPI_Send(&ready,1,MPI_INT,motherrank,READY,MPI_COMM_WORLD);
		MPI_Recv(&snp,1,MPI_INT,motherrank,NP_TAG,MPI_COMM_WORLD,&cstatus);
		while(snp != finish){
			long numstack;
			snend = MAX(snend,snp);
			printf(" receiving particles %d ",snp); fflush(stdout);
			sbp = Realloc(sbp,sizeof(basicparticletype)*snend);
			ptl2halonum = Realloc(ptl2halonum,sizeof(lint)*snend);
			MPI_Recv(sbp,sizeof(basicparticletype)*snp,MPI_BYTE,motherrank,R_TAG,MPI_COMM_WORLD,&cstatus);

			numstack = CurMemStack();
			mpeak = subhalo_den(sbp,snp,ptl2halonum);
			InitialOldMemStack(numstack);
			printf("End.initial send"); fflush(stdout);

			ready = WRITING;
			MPI_Issend(&ready,1,MPI_INT,motherrank,READY,MPI_COMM_WORLD, &request);
			MPI_Wait(&request,&mstatus);
			MPI_Send(&snp,1,MPI_INT,motherrank,NP_TAG,MPI_COMM_WORLD);
			MPI_Send(sbp,sizeof(basicparticletype)*snp,MPI_BYTE,motherrank,R_TAG, MPI_COMM_WORLD);
			MPI_Send(ptl2halonum,snp,MPI_INT,motherrank,PT2H_TAG, MPI_COMM_WORLD);
			MPI_Send(&mpeak,1,MPI_INT,motherrank,MPEAK_TAG,MPI_COMM_WORLD);
			printf("sent\n "); fflush(stdout);

			ready = READY;
			MPI_Send(&ready,1,MPI_INT,motherrank,READY,MPI_COMM_WORLD);
			MPI_Recv(&snp,1,MPI_INT,motherrank,NP_TAG,MPI_COMM_WORLD,&cstatus);
			if(snend > 2*INIT_NP) snend = 2*INIT_NP;
		}
	}
	if(myid == motherrank) fprintf(stderr,"End of calculation. Closing\n");
	MPI_Barrier(MPI_COMM_WORLD);
	MPI_Finalize();
	return 0;
}
void bin2ascii(struct PtlPos *,struct PtlVel *,lint );
void write_data(basicparticletype *ssbp,lint *rptl2halonum,int rrnp){
	struct PtlPos *wr;
	struct PtlVel *wvr;
	indxtype *windx;
	int wnp;
	int i,j,k;

	if (mpeak > 0){
        fwrite(&mpeak,sizeof(int),1,wrp);
		wr = (struct PtlPos *)Malloc(sizeof(struct PtlPos)*rrnp,PPTR(wr));
		wvr = (struct PtlVel *)Malloc(sizeof(struct PtlVel)*rrnp,PPTR(wvr));
		windx = (indxtype *)Malloc(sizeof(indxtype)*rrnp,PPTR(windx));
		for(i=0;i<mpeak;i++){
			wnp = 0;
			for(j=0;j<rrnp;j++){
				if(rptl2halonum[j] == i){
					wr[wnp].x = ssbp[j].x*size/(float) ng;
					wr[wnp].y = ssbp[j].y*size/(float) ng;
					wr[wnp].z = ssbp[j].z*size/(float) ng;
					wvr[wnp].vx = ssbp[j].vx*r2kineticfact/100000.;
					wvr[wnp].vy = ssbp[j].vy*r2kineticfact/100000.;
					wvr[wnp].vz = ssbp[j].vz*r2kineticfact/100000.;
					windx[wnp] = ssbp[j].indx;
					wnp++;
				}
			}
			fwrite(&wnp,sizeof(lint),1,wrp);
			fwrite(wr,sizeof(struct PtlPos),wnp,wrp);
			fwrite(wvr,sizeof(struct PtlVel),wnp,wrp);
			fwrite(windx,sizeof(indxtype),wnp,wrp);
#ifdef CHECK_HALO_SAVING
			fprintf(stdout,"in %d halos ,halomass %12G %d \n",mpeak,pntmass*wnp/onesolarmass,wnp);
			fflush(stdout);
#endif
		}
		i = NOT_HALO_MEMBER;
		{
			wnp = 0;
			for(j=0;j<rrnp;j++){
				if(rptl2halonum[j] == i){
					wr[wnp].x = ssbp[j].x*size/(float) ng;
					wr[wnp].y = ssbp[j].y*size/(float) ng;
					wr[wnp].z = ssbp[j].z*size/(float) ng;
					wvr[wnp].vx = ssbp[j].vx*r2kineticfact/100000.;
					wvr[wnp].vy = ssbp[j].vy*r2kineticfact/100000.;
					wvr[wnp].vz = ssbp[j].vz*r2kineticfact/100000.;
					windx[wnp] = ssbp[j].indx;
					wnp++;
				}
			}
			fwrite(&wnp,sizeof(lint),1,wbp);
			fwrite(wr,sizeof(struct PtlPos),wnp,wbp);
			fwrite(wvr,sizeof(struct PtlVel),wnp,wbp);
			fwrite(windx,sizeof(indxtype),wnp,wbp);
		}
		Free(windx);
		Free(wvr);
		Free(wr);
	}
#ifdef CHECK_HALO_SAVING
	if(myid == 0 && mpeak >0 ) {
		fprintf(stderr,"saved.\n");
	}
#endif
#ifdef CHECK_HALO
    bin2ascii(rrx,rrvx,rrnp);
#endif
	fflush(wrp);
	fflush(wbp);

}
void minmax(struct PtlPos *,int ,float *,float *, float *,float *,float *, float *,float *);
void bin2ascii(struct PtlPos *r,struct PtlVel *vr,lint np)
{
	int i,j;
	FILE *fp,*fpt,*fptz;
	char char1;
	float minx,minz,maxx,maxz,miny,maxy,width;
	int uline,dline;
	char *ctype[] = {"yellow", "blue", "red","green","6",
	"magenta","6","7","8","5","4","3","0","yellow","blue","red",
	"magenta","6","7","8","5","4","3","0","yellow","blue","red",
	"magenta","6","7","8","5","4","3","0","yellow","blue","red",
	"magenta","6","7","8","5","4","3","0","yellow","blue","red",
	"magenta","6","7","8","5","4","3","0","yellow","blue","red",
	"magenta","6","7","8","5","4","3","0","yellow","blue","red",
	"magenta","6","7","8","5","4","3","0","yellow","blue","red" };
	fp = fopen("halo0.dat","w");
	fpt = fopen("r.sm","w");
	fptz = fopen("z.sm","w");
	fprintf(fpt,"erase \n");
	fprintf(fptz,"erase \n");
	fprintf(fpt,"ticksize 0.1 1 0.1 1 \n");
	fprintf(fptz,"ticksize 0.1 1 0.1 1 \n");
	fprintf(fpt,"data halo0.dat \n");
	fprintf(fptz,"data halo0.dat \n");
	fprintf(fpt," ptype 0 0 \n");
	fprintf(fptz," ptype 0 0 \n");
	minmax(r,np,&minx,&maxx,&miny,&maxy,&minz,&maxz,&width);
	minx = floor(minx);minz = floor(minz);width = ceil(width)+1;
	fprintf(fpt," limits %f %f %f %f \n",minx,minx+width,minz,minz+width);
	fprintf(fptz," limits %f %f %f %f \n",minx,minx+width,miny,miny+width);
	fprintf(fpt," ctype default \n");
	fprintf(fptz," ctype default \n");
	fprintf(fpt," expand 1.2 \n");
	fprintf(fptz," expand 1.2 \n");
	fprintf(fpt," box \n");
	fprintf(fptz," box \n");
	fprintf(fpt," expand 0.3 \n");
	fprintf(fptz," expand 0.3 \n");
	uline = 1;dline = 0;
	for(j=0;j<mpeak;j++){
		for(i=0;i<np;i++){
			if(ptl2halonum[i] == j){
			fprintf(fp,"%f %f %f %f %f %f \n",r[i].x,r[i].y,r[i].z,
					vr[i].vx,vr[i].vy,vr[i].vz);
			dline++;
			}
		}
		fflush(fp);
		if(dline > uline ){
		fprintf(fpt," lines %d %d \n",uline,dline);
		fprintf(fptz," lines %d %d \n",uline,dline);
		fprintf(fpt," read { x 1 y 2 z 3} \n");
		fprintf(fptz," read { x 1 y 2 z 3} \n");
  		fprintf(fpt," ctype %s \n",*(ctype+j));
  		fprintf(fptz," ctype %s \n",*(ctype+j));
		fprintf(fpt," points x z \n");
		fprintf(fptz," points x y \n");
		fflush(fpt);
		}
		uline = dline + 1;
	}
	/* ̹  ׶ ƼŬ̴. */
	uline = dline;
	for(i=0;i<np;i++){
		if(ptl2halonum[i]<0){
			fprintf(fp,"%f %f %f %f %f %f \n",r[i].x,r[i].y,r[i].z,
					vr[i].vx,vr[i].vy,vr[i].vz);
			dline++;
		}
	}
	fprintf(fpt," lines %d %d \n",uline,dline);
	fprintf(fptz," lines %d %d \n",uline,dline);
	fprintf(fpt," read { x 1 y 2 z 3} \n");
	fprintf(fptz," read { x 1 y 2 z 3} \n");
  	fprintf(fpt," ctype default \n");
  	fprintf(fptz," ctype default \n");
	fprintf(fpt," points x z \n");
	fprintf(fptz," points x y \n");
	fflush(fpt);
	fflush(fptz);
	fclose(fpt);
	fclose(fptz);
	fflush(fp);
	fclose(fp);
	printf("input arbitrary value : ");
	char1 = getc(stdin);
	fflush(stdin);
}

void minmax(struct PtlPos *r,int np,float *minx,float *maxx,
		float *miny,float *maxy,float *minz,
		float *maxz,float *width){
	int i;
	*minx = *miny = *minz = 1000;
	*maxx = *maxy = *maxz = 0;
	for(i=0;i<np;i++){
		*minx = MIN(*minx,r[i].x);
		*miny = MIN(*miny,r[i].y);
		*minz = MIN(*minz,r[i].z);
		*maxx = MAX(*maxx,r[i].x);
		*maxy = MAX(*maxy,r[i].y);
		*maxz = MAX(*maxz,r[i].z);
	}
	*width = MAX(*maxx-*minx,*maxz-*minz);
	*width = MAX(*width,*maxy-*miny);
	*width = MAX(*width,4.);
}

int physical_parameters(void){
	float omepk;
	float Hsub;
	omepk = 1.-omep-omeplam;
	Hsub = sqrt(omep*pow3(amax/a)+omeplam+omepk*pow2(amax/a));
	/*    Ѵ. */
	com2real = size/hubble/rng/amax*a*1.E6L*pc;
	real2com = 1.L/com2real;
	/*
  	pntmass = 3./8./pi/G*pow2(100.*100000.*hubble)*pow(10.,6)*pc;
  	*/
  	pntmass = 3.L/8.L/pi/G*pow2(100.L*100000.L*hubble)*1.E6L*pc;
	pntmass = pntmass*pow3(size/hubble)/pow3(rng/nspace)*omep;
	potentfact = G/acoeff[0]*pntmass/com2real;
	/* V(r) = \dot a/ a_max *x + a/a_max * \dot x                     */
	/*      = H /(1+z) * x + u/(1+z) * \dot a                         */
	/*      = H /(1+z) * x + u/(1+z) * H * a                          */
	/*  ׷ x   comoving scale  distance ̱ 
	 *  x = x(pm) * L{size} / rng ( rng = one dimentional mesh size)
	 *  ̰, 
	 *  u = u(pm) * L{size} / rng ̾ ϴµ, ùķ̼ fda 
	 *   ̹, u(pm) ȿ rng   ִ... ڵ忡 pfact 
	 *     astep  ־ ϴµ, rng  ִ.
	 *   ̰ velocity  ̹ rng   ִٴ  Ѵ. ^^*/
	/* 10.24*h^-1/(amax/a)*100km/s/Mpc*h/pixel_size*(1+z)^1.5 
	r1kineticfact = size/hubble/rng/amax*a*100.E5*hubble*pow(amax/a,1.5); */
	/* 10.24*h^-1/(amax/a)*a*100km/s/Mpc*h*(1+z)^1.5 
	r2kineticfact = size/hubble/amax*a*a*100.E5*hubble*pow(amax/a,1.5); */
	/*
	 * X : in real scale, dX/dt : peculiar velocity in real scale
	 * dx : displacement in simulation dimension.
	 * da : simulation time step
	 * dx/da : velocity in simulation dimensions
	 * dX/dt = \dot{a} dX/da
	 *       \dot{a} = a \times H_o \times sqrt{Omega(1+z)^3+...}
	 *       dX/da = Lsize*a/amax dx/da
	 */
	r1kineticfact = size/hubble/rng/amax*a*100.E5L*hubble*Hsub;
	r2kineticfact = size/hubble/amax*a*a*100.E5L*hubble*Hsub;
	if(myid ==0){
		fprintf(stdout,"P%d has omep=%g rng=%g r1kineticfact=%g r2kineticfact=%g potentfact=%g\n",myid,omep,rng,r1kineticfact,r2kineticfact,potentfact);
		fflush(stdout);
	}
	return 0;
}
